package com.telpo.dipperposition.common;

import lombok.extern.slf4j.Slf4j;

import java.io.*;
import java.net.Socket;
/**
 * @program: dipperposition
 * @description: socket连接单元
 * @author: king
 * @create: 2021-01-14 13:52
 **/
@Slf4j
public class SocketClient {
    //定义一个Socket对象
    Socket socket = null;

    public SocketClient(String host, int port, int timeout) {
        try {
            //需要服务器的IP地址和端口号，才能获得正确的Socket对象
            socket = new Socket(host, port);
            socket.setSoTimeout(timeout);
        } catch (IOException e) {
           log.error("Socket Connect Error:" + e.getMessage());
        }
    }

    public String getOutput() {
        try {
            OutputStream os = socket.getOutputStream();
            return os.toString();
        } catch (IOException e) {
            log.error("Socket getOutputStream Error:" + e.getMessage());
            return null;
        }
    }


    public String sendCmd(String astCmd, String ackAckCheckRef) {
        try {
            OutputStream os=socket.getOutputStream();
            PrintWriter pw=new PrintWriter(os);
            // TODO 发生命令
            //String info="用户名：Tom,用户密码：123456";
            pw.write(astCmd);
            pw.flush();
            socket.shutdownOutput();

            //接收服务器的相应
            String reply=null;
            //输入流
            InputStream is=socket.getInputStream();
            BufferedReader br=new BufferedReader(new InputStreamReader(is));

            String ackResult="";
            String ackHexOut = HexConvert.convertStringToHex(ackAckCheckRef);
            while(!((reply=br.readLine())==null)){
                log.debug("接收服务器的信息："+reply);
                if (ackHexOut.equals(reply)) {
                    reply=br.readLine();
                    ackResult = HexConvert.convertHexToString(reply);
                    break;
                }
            }
            //4.关闭资源
            br.close();
            is.close();
            pw.close();
            os.close();
            log.debug(ackResult);
            return ackResult;
        } catch (IOException e) {
            log.error("Socket sendCmd Error:" + e.getMessage());
            return null;
        }
    }

    public void closeConnection() {
        try {
            socket.close();
            //socket.shutdownOutput();
        } catch (IOException e) {
            log.error("Socket getOutputStream Error:" + e.getMessage());
        }
    }
}
